// This program demonstrates how to call fp_exec and process the output

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "exit_codes.h"

const char * device_name = "TremolFP";
const char * log_file = "fp_demo.log";

char out[1024];
char cmd[1024];

int main( int argc, char ** argv )
{
	char cnt[32];

	cnt[0] = 0;
	if ( argc != 2 )
	{
		printf( "Usage: %s <input_file.xml>\n\n", argv[0] );
		exit(1);
	}
cont:
	sprintf( cmd, "./fp_exec -d %s -l \"%s\" -f \"%s\" %s 2>&1", device_name, log_file, argv[1], cnt );

	FILE * f = popen( cmd, "r" );
	int t = 0;
	memset( out, 0, sizeof out );
	while ( !feof( f ) )
	{
		// Here you can catch progress info (looking for '\r') and display graphical progress bar.
		int c = fgetc( f );
		if ( c > 0 && c < 256 )
		{
			if ( t < sizeof out )
			{
				out[t++] = (char)c;
				if ( c == 10 || c == 13 )
				{
					out[t - 1] = 0;
					t = 0;
				}
			}
			putc( c, stdout );
			if ( c == '\r' ) fflush( stdout );
		}
	}
	int res = pclose( f );

	if ( WEXITSTATUS( res ) == EXIT_FP_ERR )
	{
		for ( int i = 0; i < sizeof out - 2 && out[i]; i++ )
		{
			if ( out[i] == '{' && ( out[i + 2] == '}' || out[i + 3] == '}' || out[i + 4] == '}' || out[i + 5] == '}' ) )
			{
				char * eos = NULL;

				// When exit code is EXIT_FP_ERR and error message contains {?}, the command can be retried.
				// Parameter "-c N" (continue from N) will not send first N commands to device.
				sprintf( cnt, "-c %ld", strtol( out + i + 1, &eos, 0 ) );
				if ( eos && *eos == '}' )
				{
unknown_answer:
					printf( "\nRetry or Cancel [Y/n]? " );
					*cmd = 0;
					fgets( cmd, sizeof cmd, stdin );
					switch ( toupper( *cmd ) )
					{
						case 10:
						case 'R':
						case 'Y':
							goto cont;

						case 'A':
						case 'C':
						case 'N':
							break;

						default:
							goto unknown_answer;
					}
				}
			}
		}
	}
	printf( "\n[ fp_exec exit code = %d ]\n\n", WEXITSTATUS( res ) );

	return 0;
}
